package com.persagy.proxy.adm.utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.ResourceUtils;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.request.AdmResponse;

import lombok.extern.slf4j.Slf4j;

/**
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年10月21日 下午12:24:28
 */
@Slf4j
public class AdmExcelUtil {

	/**
	 * 文件写入至本地
	 * 
	 * @param workbook
	 * @param projectId
	 * @param orgFileName
	 * @return 
	 */
	public static String writeFile2Local(XSSFWorkbook workbook, String projectId, String orgFileName) {
		FileOutputStream fileOutputStream = null;
		String date = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));
		// 重组文件名
        String fileName = projectId + date + "." + orgFileName.substring(orgFileName.lastIndexOf(".") + 1);
        try {
    		// 调用中台存储完成之后，此文件存储至本地
    		File templateFile = createTemplateFile(fileName);
    		fileOutputStream = new FileOutputStream(templateFile);
    		workbook.write(fileOutputStream);
		} catch (Exception e) {
			log.error("文件存储失败", e);
		} finally {
			if (fileOutputStream != null) {
				try {
					fileOutputStream.close();
				} catch (IOException e) {}
			}
		}
        return fileName;
	}
	
	/**
	 * 获取对应的文件，或者默认文件
	 * 不同路径，防止镜像部署时获取不到
	 * 
	 * @param templateName 带后缀的文件名
	 * @return
	 * @throws FileNotFoundException 
	 */
	public static File getTemplateOrDefaultFile(String templateName) throws FileNotFoundException {
		File file = null;
		try {
			file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "template/" + templateName);
		} catch (FileNotFoundException e) {
			file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "template/" + templateName);
		}
		if (file == null || !file.exists()) {
			try {
				file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "template/relation-template.xlsx");
			} catch (FileNotFoundException e) {
				file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "/template/relation-template.xlsx");
			}
		}
		return file;
	}
	
	/**
	 * 获取对应的模板文件，不同路径，防止镜像内获取不到
	 * 
	 * @param templateName 带后缀的文件名
	 * @return
	 * @throws FileNotFoundException 
	 */
	public static File getTemplateFile(String templateName) throws FileNotFoundException {
		File file = null;
		try {
			file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "template/" + templateName);
		} catch (FileNotFoundException e) {}
		if (file == null || !file.exists()) {
			file = ResourceUtils.getFile(AdmCommonConstant.SERVER_ROOT_PATH + "/template/" + templateName);
		}
		return file;
	}
	
	/**
	 * 创建本地文件，这里不再在classpath下创建
	 * 
	 * @param uploadFile 文件名称，带后缀
	 * @throws IOException 
	 */
	public static File createTemplateFile(String uploadFile) throws IOException {
		String path = AdmCommonConstant.SERVER_ROOT_PATH + "/template";
		File file = new File(path);
		if (!file.exists()) {
			file.mkdirs();
		}
		file = new File(path + "/" + uploadFile);
		if (!file.exists()) {
			file.createNewFile();
		}
		return file;
	}
	
	/**
	 * 获取导入结果
	 * 
	 * @return
	 */
	public static JSONObject getImportResult(String graphId, int successNum, int failureNum) {
		JSONObject result = new JSONObject();
		result.put("relationType", graphId);
		result.put("successCount", successNum);
		result.put("failCount", failureNum);
		String state = null;
        if (successNum > 0 && failureNum > 0) {
        	state = "1";
        } else if (successNum > 0 && failureNum == 0) {
            state = "0";
        } else {
            state = "2";
        }
        result.put("state", state);
		return result;
	}
	
	/**
	 * 获取导入结果
	 * 
	 * @param message
	 * @param successCount
	 * @param failCount
	 * @return
	 */
	public static AdmResponse getImportFailResult(String message, String state, int successCount, int failCount) {
		JSONObject content = new JSONObject();
		content.put("state", state);
		content.put("successCount", successCount);
		content.put("failCount", failCount);
		
		AdmResponse failure = AdmResponse.failure(message);
		failure.setContent(Lists.newArrayList(content));
		return failure;
	}
	
	/**
	 * 获取导入结果
	 * 
	 * @param successCount
	 * @param failCount
	 * @return
	 */
	public static AdmResponse getImportSuccessResult(String message, int successCount, int failCount) {
		JSONObject content = new JSONObject();
		String state = null;
        if (successCount > 0 && failCount > 0) {
        	state = "1";
        } else if (successCount > 0 && failCount == 0) {
            state = "0";
        } else {
            state = "2";
        }
		content.put("state", state);
		content.put("successCount", successCount);
		content.put("failCount", failCount);
		AdmResponse success = AdmResponse.success(Lists.newArrayList(content));
		success.setMessage(message);
		return success;
	}
	
}
